@@ -1,21 +1,15 @@
 // OPCUA for Rust
 // SPDX-License-Identifier: MPL-2.0
-// Copyright (C) 2017-2020 Adam Lock
+// Copyright (C) 2017-2022 Adam Lock
 
 use opcua_types::*;
 
-use crate::{
-    address_space::address_space::AddressSpace,
-    events::event::Event,
-};
+use crate::{address_space::address_space::AddressSpace, events::event::Event};
 
-use super::{
-    AuditEvent,
-    event::AuditEventType,
-};
+use super::{event::AuditEventType, AuditEvent};
 
 pub struct AuditNodeManagementEventType {
-    base: AuditEventType
+    base: AuditEventType,
 }
 
 impl Event for AuditNodeManagementEventType {
